
' <copyright file="IAssetExtensions.cs" company="Microsoft">Copyright 2013 Microsoft Corporation</copyright>
' <license>
' Licensed under the Apache License, Version 2.0 (the "License");
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' 
' http://www.apache.org/licenses/LICENSE-2.0
' 
' Unless required by applicable law or agreed to in writing, software
' distributed under the License is distributed on an "AS IS" BASIS,
' WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
' See the License for the specific language governing permissions and
' limitations under the License.
' </license>

Imports System.Collections.Generic
Imports System.Globalization
Imports System.IO
Imports System.Linq
Imports System.Threading
Imports System.Threading.Tasks
Namespace Microsoft.WindowsAzure.MediaServices.Client

	''' <summary>
	''' Contains extension methods and helpers for the <see cref="IAsset"/> and <see cref="IAssetFiles"/> interfaces.
	''' </summary>
    Public Module IAssetExtensions

        ''' <summary>
        ''' Returns a <see cref="System.Threading.Tasks.Task"/> instance to generate <see cref="IAssetFile"/> for the <paramref name="asset"/>.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance where to generate its <see cref="IAssetFile"/>.</param>
        ''' <returns>A <see cref="System.Threading.Tasks.Task"/> to generate <see cref="IAssetFile"/>.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Async Function GenerateFromStorageAsync(asset As IAsset) As Task
            If asset Is Nothing Then
                Throw New ArgumentNullException("asset", "The asset cannot be null.")
            End If

            Dim context As MediaContextBase = asset.GetMediaContext()

            Dim uriCreateFileInfos As New Uri(String.Format(CultureInfo.InvariantCulture, "/CreateFileInfos?assetid='{0}'", asset.Id), UriKind.Relative)

            Await context.MediaServicesClassFactory.CreateDataServiceContext().ExecuteAsync(uriCreateFileInfos, Nothing, "GET")
        End Function

        ''' <summary>
        ''' Generates <see cref="IAssetFile"/> for the <paramref name="asset"/>.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance where to generate its <see cref="IAssetFile"/>.</param>
        <System.Runtime.CompilerServices.Extension> _
        Public Sub GenerateFromStorage(asset As IAsset)
            Using task As Task = asset.GenerateFromStorageAsync()
                task.Wait()
            End Using
        End Sub

        '''// <summary>
        '''// Returns a <see cref="System.Threading.Tasks.Task"/> instance to download all the asset files in the <paramref name="asset"/> to the <paramref name="folderPath"/>.
        '''// </summary>
        '''// <param name="asset">The <see cref="IAsset"/> instance where to download the asset files.</param>
        '''// <param name="folderPath">The path to the folder where to download the asset files in the <paramref name="asset"/>.</param>
        '''// <param name="downloadProgressChangedCallback">A callback to report download progress for each asset file in the <paramref name="asset"/>.</param>
        '''// <param name="cancellationToken">The <see cref="System.Threading.CancellationToken"/> instance used for cancellation.</param>
        '''// <returns>A <see cref="System.Threading.Tasks.Task"/> instance to download all the asset files in the <paramref name="asset"/>.</returns>
        'public static async Task DownloadToFolderAsync(this IAsset asset, string folderPath, Action<IAssetFile, DownloadProgressChangedEventArgs> downloadProgressChangedCallback, CancellationToken cancellationToken)
        '{
        '    if (asset == null)
        '    {
        '        throw new ArgumentNullException("asset", "The asset cannot be null.");
        '    }

        '    if (!Directory.Exists(folderPath))
        '    {
        '        throw new ArgumentException(
        '            string.Format(CultureInfo.InvariantCulture, "The folder '{0}' does not exist.", folderPath),
        '            "folderPath");
        '    }

        '    MediaContextBase context = asset.GetMediaContext();

        '    ILocator sasLocator = await context.Locators.CreateAsync(LocatorType.Sas, asset, AccessPermissions.Read, AssetBaseCollectionExtensions.DefaultAccessPolicyDuration);

        '    EventHandler<DownloadProgressChangedEventArgs> downloadProgressChangedHandler =
        '        (s, e) =>
        '        {
        '            IAssetFile assetFile = (IAssetFile)s;
        '            DownloadProgressChangedEventArgs eventArgs = e;

        '            if (downloadProgressChangedCallback != null)
        '            {
        '                downloadProgressChangedCallback(assetFile, eventArgs);
        '            }
        '        };

        '    List<Task> downloadTasks = new List<Task>();
        '    List<IAssetFile> assetFiles = asset.AssetFiles.ToList();
        '    foreach (IAssetFile assetFile in assetFiles)
        '    {
        '        string localDownloadPath = Path.Combine(folderPath, assetFile.Name);
        '        BlobTransferClient blobTransferClient = new BlobTransferClient
        '        {
        '            NumberOfConcurrentTransfers = context.NumberOfConcurrentTransfers,
        '            ParallelTransferThreadCount = context.ParallelTransferThreadCount
        '        };

        '        assetFile.DownloadProgressChanged += downloadProgressChangedHandler;

        '        downloadTasks.Add(
        '            assetFile.DownloadAsync(Path.GetFullPath(localDownloadPath), blobTransferClient, sasLocator, cancellationToken));
        '    }

        '    await Task.Factory.ContinueWhenAll(downloadTasks.ToArray(), t => t, TaskContinuationOptions.ExecuteSynchronously);

        '    await sasLocator.DeleteAsync();

        '    assetFiles.ForEach(af => af.DownloadProgressChanged -= downloadProgressChangedHandler);
        '}

        ''' <summary>
        ''' Returns a <see cref="System.Threading.Tasks.Task"/> instance to download all the asset files in the <paramref name="asset"/> to the <paramref name="folderPath"/>.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance where to download the asset files.</param>
        ''' <param name="folderPath">The path to the folder where to download the asset files in the <paramref name="asset"/>.</param>
        ''' <param name="cancellationToken">The <see cref="System.Threading.CancellationToken"/> instance used for cancellation.</param>
        ''' <returns>A <see cref="System.Threading.Tasks.Task"/> instance to download all the asset files in the <paramref name="asset"/>.</returns>
        'public static Task DownloadToFolderAsync(this IAsset asset, string folderPath, CancellationToken cancellationToken)
        '{
        '    return asset.DownloadToFolderAsync(folderPath, null, cancellationToken);
        '}

        ''' <summary>
        ''' Downloads all the asset files in the <paramref name="asset"/> to the <paramref name="folderPath"/>.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance where to download the asset files.</param>
        ''' <param name="folderPath">The path to the folder where to download the asset files in the <paramref name="asset"/>.</param>
        ''' <param name="downloadProgressChangedCallback">A callback to report download progress for each asset file in the <paramref name="asset"/>.</param>
        'public static void DownloadToFolder(this IAsset asset, string folderPath, Action<IAssetFile, DownloadProgressChangedEventArgs> downloadProgressChangedCallback)
        '{
        '    using (Task task = asset.DownloadToFolderAsync(folderPath, downloadProgressChangedCallback, CancellationToken.None))
        '    {
        '        task.Wait();
        '    }
        '}

        ''' <summary>
        ''' Downloads all the asset files in the <paramref name="asset"/> to the <paramref name="folderPath"/>.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance where to download the asset files.</param>
        ''' <param name="folderPath">The path to the folder where to download the asset files in the <paramref name="asset"/>.</param>
        'public static void DownloadToFolder(this IAsset asset, string folderPath)
        '{
        '    asset.DownloadToFolder(folderPath, null);
        '}

        ''' <summary>
        ''' Returns the <see cref="IAssetFile"/> instance that represents the manifest file of the asset; otherwise, null.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance.</param>
        ''' <returns>A <see cref="IAssetFile"/> instance that represents the manifest file of the asset; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetManifestAssetFile(asset As IAsset) As IAssetFile
            If asset Is Nothing Then
                Throw New ArgumentNullException("asset", "The asset cannot be null.")
            End If

            Return asset.AssetFiles.ToList().Where(Function(af) af.Name.EndsWith(ILocatorExtensions.ManifestFileExtension, StringComparison.OrdinalIgnoreCase)).FirstOrDefault()
        End Function

        ''' <summary>
        ''' Returns the Smooth Streaming URL of the <paramref name="asset"/> using the on-demand origin locator with the longest expiration time; otherwise, null.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the Smooth Streaming URL of the <paramref name="asset"/>; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetSmoothStreamingUri(asset As IAsset) As Uri
            Return asset.GetStreamingUri(String.Empty)
        End Function

        ''' <summary>
        ''' Returns the HLS URL of the <paramref name="asset"/> using the on-demand origin locator with the longest expiration time; otherwise, null.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the HLS URL of the <paramref name="asset"/>; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetHlsUri(asset As IAsset) As Uri
            Return asset.GetStreamingUri(ILocatorExtensions.HlsStreamingParameter)
        End Function

        ''' <summary>
        ''' Returns the MPEG-DASH URL of the <paramref name="asset"/> using the on-demand origin locator with the longest expiration time; otherwise, null.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the MPEG-DASH URL of the <paramref name="asset"/>; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetMpegDashUri(asset As IAsset) As Uri
            Return asset.GetStreamingUri(ILocatorExtensions.MpegDashStreamingParameter)
        End Function

        ''' <summary>
        ''' Returns the SAS URL of the <paramref name="assetFile"/> for progressive download using the SAS locator with the longest expiration time; otherwise, null.
        ''' </summary>
        ''' <param name="assetFile">The <see cref="IAssetFile"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the SAS URL of the <paramref name="assetFile"/> for progressive download; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetSasUri(assetFile As IAssetFile) As Uri
            If assetFile Is Nothing Then
                Throw New ArgumentNullException("assetFile", "The asset file cannot be null.")
            End If

            Dim sasUri As Uri = Nothing
            Dim asset As IAsset = assetFile.Asset
            If asset IsNot Nothing Then
                Dim sasLocator As ILocator = asset.Locators.ToList().Where(Function(l) l.Type = LocatorType.Sas).OrderBy(Function(l) l.ExpirationDateTime).LastOrDefault()
                If sasLocator IsNot Nothing Then
                    sasUri = BuildSasUri(assetFile, sasLocator)
                End If
            End If

            Return sasUri
        End Function

        ''' <summary>
        ''' Returns the SAS URL of the <paramref name="assetFile"/> for progressive download using the <paramref name="sasLocator"/>.
        ''' </summary>
        ''' <param name="assetFile">The <see cref="IAssetFile"/> instance.</param>
        ''' <param name="sasLocator">The <see cref="ILocator"/> instance.</param>
        ''' <returns>A <see cref="System.Uri"/> representing the SAS URL of the <paramref name="assetFile"/> for progressive download; otherwise, null.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetSasUri(assetFile As IAssetFile, sasLocator As ILocator) As Uri
            If assetFile Is Nothing Then
                Throw New ArgumentNullException("assetFile", "The asset file cannot be null.")
            End If

            If sasLocator Is Nothing Then
                Throw New ArgumentNullException("sasLocator", "The SAS locator cannot be null.")
            End If

            If sasLocator.Type <> LocatorType.Sas Then
                Throw New ArgumentException("The locator type must be SAS.", "sasLocator")
            End If

            If assetFile.ParentAssetId <> sasLocator.AssetId Then
                Throw New ArgumentException("sasLocator", "The SAS locator does not belong to the parent asset.")
            End If

            Dim sasUri As Uri = BuildSasUri(assetFile, sasLocator)

            Return sasUri
        End Function

        ''' <summary>
        ''' Returns the parent <see cref="MediaContextBase"/> instance.
        ''' </summary>
        ''' <param name="asset">The <see cref="IAsset"/> instance.</param>
        ''' <returns>The parent <see cref="MediaContextBase"/> instance.</returns>
        <System.Runtime.CompilerServices.Extension> _
        Public Function GetMediaContext(asset As IAsset) As MediaContextBase
            If asset Is Nothing Then
                Throw New ArgumentNullException("asset", "The asset cannot be null.")
            End If

            Dim mediaContextContainer As IMediaContextContainer = TryCast(asset, IMediaContextContainer)
            Dim context As MediaContextBase = Nothing

            If mediaContextContainer IsNot Nothing Then
                context = mediaContextContainer.GetMediaContext()
            End If

            Return context
        End Function

        <System.Runtime.CompilerServices.Extension> _
        Friend Async Function CreateAssetFileFromLocalFileAsync(asset As IAsset, filePath As String, sasLocator As ILocator, _uploadProgressChangedEventArgs As EventHandler(Of UploadProgressChangedEventArgs), _cancellationToken As CancellationToken) As Task(Of IAssetFile)
            Dim assetFileName As String = Path.GetFileName(filePath)
            Dim assetFile As IAssetFile = Await asset.AssetFiles.CreateAsync(assetFileName, _cancellationToken)
            Dim context As MediaContextBase = asset.GetMediaContext()

            'RaiseEvent UploadProgressChanged(_uploadProgressChangedEventArgs)
            Dim _blobTransferClient As New BlobTransferClient() With { _
                .NumberOfConcurrentTransfers = context.NumberOfConcurrentTransfers, _
                .ParallelTransferThreadCount = context.ParallelTransferThreadCount _
            }

            Await assetFile.UploadAsync(filePath, _blobTransferClient, sasLocator, _cancellationToken)

            'assetFile.UploadProgressChanged -= UploadProgressChangedEventArgs

            If assetFileName.EndsWith(ILocatorExtensions.ManifestFileExtension, StringComparison.OrdinalIgnoreCase) Then
                assetFile.IsPrimary = True
                Await assetFile.UpdateAsync()
            End If

            Return assetFile
        End Function

        <System.Runtime.CompilerServices.Extension> _
        Private Function GetStreamingUri(asset As IAsset, streamingParameter As String) As Uri
            If asset Is Nothing Then
                Throw New ArgumentNullException("asset", "The asset cannot be null.")
            End If

            Dim smoothStreamingUri As Uri = Nothing
            Dim manifestAssetFile As IAssetFile = asset.GetManifestAssetFile()
            If manifestAssetFile IsNot Nothing Then
                Dim originLocator As ILocator = asset.Locators.ToList().Where(Function(l) l.Type = LocatorType.OnDemandOrigin).OrderBy(Function(l) l.ExpirationDateTime).FirstOrDefault()
                If originLocator IsNot Nothing Then
                    smoothStreamingUri = New Uri(String.Format(CultureInfo.InvariantCulture, ILocatorExtensions.BaseStreamingUrlTemplate, originLocator.Path.TrimEnd("/"c), manifestAssetFile.Name, streamingParameter), UriKind.Absolute)
                End If
            End If

            Return smoothStreamingUri
        End Function

        Private Function BuildSasUri(assetFile As IAssetFile, sasLocator As ILocator) As Uri
            Dim builder As New UriBuilder(New Uri(sasLocator.Path, UriKind.Absolute))
            builder.Path = Path.Combine(builder.Path, assetFile.Name)

            Return builder.Uri
        End Function
    End Module
End Namespace

