
' -----------------------------------------------------------------------
' <copyright file="IBroadcastProgram.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.MediaServices.Client
Namespace LiveMediaManager

	''' <summary>
	''' Simple abstraction that allows us to add a start / date time and a duration.
	''' </summary>
	Public Interface IBroadcastProgram
		''' <summary>
		''' Gets the current program.
		''' </summary>
		ReadOnly Property Program() As IProgram

		''' <summary>
		''' Gets the program's start date and time in UTC - 0 or GMT
		''' </summary>
		ReadOnly Property StartDateTime() As DateTime

		''' <summary>
		''' Starts the program.
		''' </summary>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Function Start() As Task

		''' <summary>
		''' Stops the program.
		''' </summary>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Function [Stop]() As Task

		''' <summary>
		''' Deletes the program and associated asset.
		''' </summary>
		''' <param name="all">if true then remove assets and locators as well</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Function Delete(all As Boolean) As Task

		Function WaitForState(state As ProgramState) As Task(Of Boolean)
	End Interface
End Namespace
