
' -----------------------------------------------------------------------
' <copyright file="IBroadcastSchedule.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Imports Microsoft.WindowsAzure.MediaServices.Client
Imports System.Collections.Generic
Imports System.Threading.Tasks
Namespace LiveMediaManager

	''' <summary>
	''' Simple abstraction that allows the management of individual programs.
	''' </summary>
	Public Interface IBroadcastSchedule
		Inherits IList(Of IBroadcastProgram)
		''' <summary>
		''' Creates a program associated with this schedule.
		''' </summary>
		''' <param name="options">configuration for the program</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Function Create(options As ProgramCreationOptions) As Task(Of IBroadcastProgram)

		''' <summary>
		''' Locates a program by name.
		''' </summary>
		''' <param name="name">name of program</param>
		''' <returns>the program or null if not found</returns>
		Function Find(name As String) As IBroadcastProgram

		''' <summary>
		''' Requests to ensure all of the programs are in the requested state.
		''' </summary>
		''' <param name="state"></param>
		''' <returns>a list of running tasks</returns>
		Function WaitForAllPrograms(state As ProgramState) As List(Of Task(Of Boolean))

		''' <summary>
		''' Stops all programs associated with this schedule.
		''' </summary>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Function StopAllPrograms() As List(Of Task)

		''' <summary>
		''' Delete all programs associated with this schedule.
		''' </summary>
		''' <param name="deleteAssets">true if we should remove assets as well.</param>
		''' <returns>a task used for tracking the completion of the operation</returns>
		Function DeleteAllPrograms(Optional deleteAssets As Boolean = False) As List(Of Task)
	End Interface
End Namespace
