
' -----------------------------------------------------------------------
' <copyright file="ProgramSettings.cs" company="Microsoft">
' Copyright © Microsoft Corporation.  All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
' FITNESS FOR A PARTICULAR PURPOSE.
' </copyright>
'  <notes>
'                  \\|//
'                  (o o)
' --------------ooO-(_)-Ooo--------------------
'  Authors: Developer Product Evangelist Team (greg.prentice)
'  License: Apache License Version 2.0, January 2004
'                       Oooo
' --------------oooO----(  )-------------------
'               (  )    (_/
'                \_)
'  </notes>
' -----------------------------------------------------------------------

Namespace LiveMediaManager

	''' <summary>
	''' Defines the values used when configuring a program.
	''' </summary>
	Public Class ProgramSettings_old
		''' <summary>
		''' Initializes a new instance of the <see cref="ProgramSettings"/> class.
		''' </summary>
		''' <param name="name">name of the program</param>
		Public Sub New(name As String)
			Me.Name = name
			Me.Description = String.Empty
			Me.IsArchiveEnabled = True
			Me.DvrWindowLength = New TimeSpan(0, 2, 0)
			Me.EstimatedDuration = New TimeSpan(0, 60, 0)
		End Sub

		''' <summary>
		''' Gets or sets a unique name of the channel.
		''' </summary>
		Public Property Name() As String
			Get
				Return m_Name
			End Get
			Set
				m_Name = Value
			End Set
		End Property
		Private m_Name As String

		''' <summary>
		''' Gets or sets the description of the program.
		''' </summary>
		Public Property Description() As String
			Get
				Return m_Description
			End Get
			Set
				m_Description = Value
			End Set
		End Property
		Private m_Description As String

		''' <summary>
		''' Gets or sets a value indicating whether archival is requested.
		''' </summary>
		Public Property IsArchiveEnabled() As Boolean
			Get
				Return m_IsArchiveEnabled
			End Get
			Set
				m_IsArchiveEnabled = Value
			End Set
		End Property
		Private m_IsArchiveEnabled As Boolean

		''' <summary>
		''' Gets or sets how long of a video chunk is stored to allow a video player's DVR functionality.
		''' </summary>
		Public Property DvrWindowLength() As TimeSpan
			Get
				Return m_DvrWindowLength
			End Get
			Set
				m_DvrWindowLength = Value
			End Set
		End Property
		Private m_DvrWindowLength As TimeSpan

		''' <summary>
		''' Gets or sets an estimation of how long the video should play.
		''' </summary>
		Public Property EstimatedDuration() As TimeSpan
			Get
				Return m_EstimatedDuration
			End Get
			Set
				m_EstimatedDuration = Value
			End Set
		End Property
		Private m_EstimatedDuration As TimeSpan
	End Class
End Namespace
